/*
	pvfs_writev.c copyright (c) 1998 Clemson University, all rights reserved.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Walt Ligon walt@eng.clemson.edu

/*/

/*
 * $Log: pvfs_writev.c,v $
 * Revision 1.1  1998/07/23 21:54:06  rbross
 * Initial revision
 *
 *
/*/

static char pvfs_writev_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_writev.c,v 1.1 1998/07/23 21:54:06 rbross Exp $";

#include <lib.h>
#include <sys/uio.h>

extern fdesc_p pfds[];
extern jlist_p active_p;
extern sockset socks;

int unix_writev(int fd, const struct iovec *vector, size_t count);

int pvfs_writev(int fd, const struct iovec *vector, size_t count)
{
	int i, size = 0;
	ireq req;
	iack ack;
	fdesc_p pfd_p = pfds[fd];

	if (fd < 0 || fd >= NR_OPEN || (pfds[fd] && pfds[fd]->fs == FS_RESV)) {
		errno = EBADF;
		return(-1);
	}  

	if (!pfd_p || pfd_p->fs == FS_UNIX) return(unix_writev(fd, vector, count));

	errno = ENOSYS;
	return(-1);
}

#define OFFSET req.req.rw.off
#define FSIZE  req.req.rw.fsize
#define GSIZE  req.req.rw.gsize
#define LSIZE  req.req.rw.lsize
#define GCOUNT req.req.rw.gcount
#define STRIDE req.req.rw.stride

int unix_writev(int fd, const struct iovec *vector, size_t count)
{
	fdesc_p fd_p = pfds[fd];
	ireq req;
	int off, ret;

	/* if there's no partition, then we don't have to do any work! */
	if (!fd_p || !fd_p->part_p) return(syscall(SYS_writev, fd, vector, count));

	errno = ENOSYS;
	return(-1);

#if 0
	build_rw_req(fd_p, count, &req, IOD_RW_READ, 0);

	/* Notes: request parameters are in terms of the WHOLE file,
	 * whereas the offset stored in fd_p->fd.off is in terms of the
	 * PARTITIONED file.
	 *
	 * Because of this, we're just adding to the partition offset
	 * however much data gets read, while the offset into the whole
	 * file has to skip around a lot.
	 *
	 * We should be able to add a couple of checks to avoid some of
	 * these lseek()s.
	 */
	off = OFFSET;

	if (FSIZE) {
		if (syscall(SYS_lseek, fd, off, SEEK_SET) < 0) return(-1);
		if ((ret = syscall(SYS_read, fd, buf, FSIZE)) < 0) return(-1);
		buf += ret;
		if (ret < FSIZE) /* this is all we're getting */ {
			fd_p->fd.off += buf-orig_buf;
			return(buf-orig_buf);
		}
		off += FSIZE + STRIDE - GSIZE;
	}
	while (GCOUNT-- > 0) {
		if (syscall(SYS_lseek, fd, off, SEEK_SET) < 0) return(-1);
		if ((ret = syscall(SYS_read, fd, buf, GSIZE)) < 0) return(-1);
		buf += ret;
		if (ret < GSIZE) /* this is all we're getting */ {
			fd_p->fd.off += buf-orig_buf;
			return(buf-orig_buf);
		}
		off += STRIDE;
	}
	if (LSIZE) {
		if (syscall(SYS_lseek, fd, off, SEEK_SET) < 0) return(-1);
		if ((ret = syscall(SYS_read, fd, buf, LSIZE)) < 0) return(-1);
		buf += ret;
	}
	fd_p->fd.off += buf-orig_buf;
	return(buf-orig_buf);
#endif
}
