/*
 * iodtab.c  -  Copyright (C) 1997 Clemson University
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

 *	This function is passed the name of a directory.  It looks in the
 *	iodtab file for a line that begins with that directory name and
 *	fills in a structure for that mount point with all important info.
 *	If the name is not found it returns a null pointer, otherwise it
 *	returns a pointer to the structure.  The info is only guaranteed
 *	stable until the next call to this function.
 */

static char parse_iodtab_c[] = "$Header: /projects/pvfs/pvfs-working/mgr/RCS/iodtab.c,v 1.2 1997/10/28 18:53:43 pvfs Exp $";

/* 
 * $Log: iodtab.c,v $
 * Revision 1.2  1997/10/28 18:53:43  pvfs
 * added licence
 *
 * Revision 1.1  1997/02/26 20:48:35  rbross
 * Initial revision
 *
 */

/* UNIX INCLUDES */
#include <stdio.h>
#include <string.h>

/* PVFS INCLUDES */
#include <iodtab.h>

/* load info into statically allocated structure and string table.
/* keep pointers in the structure as indexes into the string table.
/* finally, malloc space for the part of the structure acutally used
/* plus the string table, copy from the static space, and convert the
/* indicies to actual pointers - Walt */

/* modified version of ionodeinfo and iodtabinfo only used here */

/* GLOBALS */
static struct iodtabinfobuf
{
	int fsname;
	int options;
	int metafile;
	int  nodecount;
	struct ionodeinfobuf
	{
		int hostname;
		int dirname;
	} ionode[NUMNODES];
} mountinfo;

static char infobuf[INFOSIZE];
static int infop;

/* DEFINES */
#define LOG(x);
#define LOG1(x,y);

/* FUNCTIONS */
iodtabinfo_p parse_iodtab(file_p, dir_p)
char *file_p;
char *dir_p;
{
	FILE *iodtab;
	int done = 0; /* boolean to control loops */
	char inbuf[BUFSIZE];
	char *cp;
	struct iodtabinfo *mi;

	/* first open the file */
	if ((iodtab = fopen(file_p,"r")) == NULL) {
		LOG("fopen");
		return(NULL);
	}
	/* first find the line for the requested directory */
	done = 0;
	while (!done) {
		if (!fgets(inbuf,BUFSIZE,iodtab))
			return NULL;
		if (*inbuf == '#')
			continue;
		if (!strcmp(dir_p,cp = strtok(inbuf," \t:\n")))
			done++; /* break the loop */
	}
	/* inbuf now contains the tab line for the directory */
	infop = 0; /* used as an index at first; values converted to pointers
					later */
	mountinfo.fsname = infop;
	while(infobuf[infop++] = *cp++);
	/* we now parse the remaining info */
	cp = strtok(NULL," \t:\n");
	mountinfo.options = infop;
	if (*cp == '-') {
		while(infobuf[infop++] = *cp++);
		cp = strtok(NULL," \t:\n");
	}
	else
		infobuf[infop++] = '\000';
	mountinfo.metafile = infop;
	while(infobuf[infop++] = *cp++);
	/* now we check for IO node info */
	mountinfo.nodecount = 0;
	done = 0;
	while (!done) {
		if (!fgets(inbuf,BUFSIZE,iodtab)) {
			done++; /* break the loop */
			continue;
		}
		if (*inbuf == '#')
			continue;
		if (!(cp = strtok(inbuf," \t\n")) || (*cp != '-')) {
			done++; /* break the loop */
			continue;
		}
		else {
			cp = strtok(NULL," \t:\n");
			mountinfo.ionode[mountinfo.nodecount].hostname =
				infop;
			while(infobuf[infop++] = *cp++);
			cp = strtok(NULL," \t:\n");
			mountinfo.ionode[mountinfo.nodecount].dirname =
				infop;
			while(infobuf[infop++] = *cp++);
			mountinfo.nodecount++;
		}
	}
	/* return pointer to the info */
	{
		/* compute size of structure actually used */
		int mi_size = mountinfo.nodecount * sizeof(struct ionodeinfo) +
			(sizeof(struct iodtabinfo) -
			(NUMNODES * sizeof(struct ionodeinfo)));
		char *st; /* get a pointer for the string table */
		int i; /* index variable */

		/* malloc structure + string table */
		mi = (struct iodtabinfo *)malloc(mi_size + infop);
		/* find address of string table */
		st = (char *)(mi) + mi_size;
		/* copy structure */
		/* bcopy((char *)&mountinfo,(char *)mi,mi_size);  RR*/
		mi->nodecount = mountinfo.nodecount;
		/* copy string table */
		bcopy(infobuf,st,infop);
		/* convert indicies into pointers */
		mi->fsname = &st[mountinfo.fsname];
		mi->options = &st[mountinfo.options];
		mi->metafile = &st[mountinfo.metafile];
		for (i = 0; i < mountinfo.nodecount; i++) {
			mi->ionode[i].hostname = &st[mountinfo.ionode[i].hostname];
			mi->ionode[i].dirname = &st[mountinfo.ionode[i].dirname];
		}
	}
	/* NOW return the pointer */
	return(mi);
}

void dump_iodtab(mp)
struct iodtabinfo *mp;
{
   int i;
   if (mp!=NULL) 
   {
	printf("\nfs - %s\noptions - %s\nmetafile - %s\n",
		mp->fsname,mp->options,mp->metafile);
	for(i = 0; i < mp->nodecount; i++) {
		printf("iod %d - %s:%s\n", i,
			mp->ionode[i].hostname,mp->ionode[i].dirname);
	}
	printf("\n");
   }
}
